# extended Guinier analysis
# Zheng W, and Best RB. 
# J. Mol. Biol. 16:2540 (2018)

usage='''
command line:

                ./extended.py <file for SAXS with two columns> <number of peptide bonds>

input:
                SAXS file with two columns (q and I(q))
                number of peptide bonds

output:
                an output file with six columns: qRg, q, Rg, Rg_err, nu, nu_err
                A good estimate Rg for IDPs can be selected for the line with the largest qRg when qRg<2.0
'''

import numpy as np
import matplotlib.pyplot as plt
import scipy.optimize as op
import os,sys
import matplotlib.cm as cm

def alpha_i(nu):
	return -0.0479*(nu-0.212)
def nu2rg(nu,nuerr,N):
	gamma=1.1615
	b=5.5
	rg2=gamma*(gamma+1)/2/(2*nu+gamma)/(2*nu+gamma+1)*b**2*N**(2*nu)
	nusigma=nuerr/nu
	rg2err=np.sqrt(2*nusigma**2+(2*nusigma*np.log(N))**2)*rg2
	return rg2,rg2err
def lnI_i(nu,lnI0,q2,N):
	rg2,rg2err=nu2rg(nu,0.,N)
	return -1./3*q2*rg2+lnI0-alpha_i(nu)*q2**2*rg2**2

def func_saxs1(x,q2,logc,logcerr,N):
	nu=x[0]
	lnI0=x[1]
	lnI=lnI_i(nu,lnI0,q2,N)
	return np.sum(((lnI-logc)/logcerr)**2)

def func_saxs2(x,q2,logc,N):
	nu=x[0]
	lnI0=x[1]
	lnI=lnI_i(nu,lnI0,q2,N)
	return np.sum((lnI-logc)**2)

def guinier_i(q,c,cerr,N):
	q2=q**2
	logc=np.log(c)
	logcerr=cerr/c
	# one fitting with original data
	res=op.minimize(func_saxs1,x0=[0.6,logc[0]],args=(q2,logc,logcerr,N),method='Nelder-Mead')
	newnu=res.x[0]
	newlnI0=res.x[1]
	# bootstrap
	ntrial=1000
	newnu_list=np.zeros(ntrial)
	newlnI0_list=np.zeros(ntrial)
	for i in range(ntrial):
		new_d=logc.copy()
		for j in range(len(logc)):
			new_d[j]+=np.random.normal(0,logcerr[j])
		res=op.minimize(func_saxs2,x0=[0.6,new_d[0]],args=(q2,new_d,N),method='Nelder-Mead')
		newnu_list[i]=res.x[0]
		newlnI0_list[i]=res.x[1]
	newnuerr=np.std(newnu_list)
	newlnI0err=np.std(newlnI0_list)
	newrg2,newrg2err=nu2rg(newnu,newnuerr,N)
	newrg=np.sqrt(newrg2)
	newrgerr=newrg2err/newrg2*newrg*0.5
	return newrg,newrgerr,newnu,newnuerr,newlnI0,newlnI0err

if len(sys.argv)!=3 or sys.argv[1]=='-h':
    print(usage)
    quit()

fn=sys.argv[1]
N=int(sys.argv[2])

ncut=3
qcut=0.1
qstart=0.0

fout=open('%s.extended'%(fn),'w')
tmp_saxs=np.loadtxt(fn,comments='#')
q=tmp_saxs[:,0]
c=tmp_saxs[:,1]
cerr=tmp_saxs[:,2]#/np.sqrt(30)
list1=np.where((q>=qstart) & (q<=qcut))[0]
npoints=len(list1)
res=[]

fout.write('# qRg, q, Rg (angstrom), Rg error, nu, nu error\n')
for qdx in range(ncut,npoints):
	print(qdx)
	newrg,newrgerr,newnu,newnuerr,newlnI0,newlnI0err=guinier_i(q[list1[:qdx]],c[list1[:qdx]],cerr[list1[:qdx]],N)
	fout.write('%7.3f %6.5f %8.5f %8.5f %8.5f %8.5f\n'%(newrg*q[list1[qdx-1]],q[list1[qdx-1]],newrg,newrgerr,newnu,newnuerr))
	fout.flush()
fout.close()

